""" The pdelab specific parts of the code generation process """

# Trigger some imports that are needed to have all backend implementations visible
# to the selection mechanisms
from dune.codegen.generation import (get_backend)
from dune.codegen.options import option_switch
from dune.codegen.pdelab.argument import pymbolic_coefficient
from dune.codegen.pdelab.function import pymbolic_gridfunction
from dune.codegen.pdelab.index import name_index
from dune.codegen.pdelab.spaces import (lfs_inames,
                                        )
from dune.codegen.pdelab.tensors import (pymbolic_list_tensor,
                                         pymbolic_identity,
                                         pymbolic_matrix_inverse,
                                         )


class PDELabInterface(object):
    def __init__(self):
        # The visitor instance will be registered by its init method
        self.visitor = None

    #
    # TODO: The following ones are actually entirely PDELab independent!
    # They should be placed elsewhere and be used directly in the visitor.
    #

    def component_iname(self, context=None, count=None):
        return component_iname(context=context, count=count)

    def name_index(self, ind):
        return name_index(ind)

    #
    # Local function space related generator functions
    #

    def lfs_inames(self, element, restriction, number=None, context=''):
        return lfs_inames(element, restriction, number, context)

    def initialize_function_spaces(self, expr, visitor):
        from dune.codegen.pdelab.spaces import initialize_function_spaces
        return initialize_function_spaces(expr, visitor)

    #
    # Test and trial function related generator functions
    #
    def pymbolic_gridfunction(self, coeff, restriction, grad):
        return pymbolic_gridfunction(coeff, restriction, grad)

    #
    # Tensor expression related generator functions
    #

    def pymbolic_list_tensor(self, o):
        return pymbolic_list_tensor(o, self.visitor)

    def pymbolic_identity(self, o):
        return pymbolic_identity(o)

    def pymbolic_matrix_inverse(self, o):
        return pymbolic_matrix_inverse(o, self.visitor)
